@@ -19,6 +19,7 @@ import logging
 import multiprocessing
 import shlex
 import sys
+import threading
 
 from oslo_config import cfg
 from oslo_config import types
@@ -64,6 +65,10 @@ OPTS = [
                       'recreate the current configuration. '
                       'This command must accept suitable --privsep_context '
                       'and --privsep_sock_path arguments.')),
+    cfg.StrOpt('logger_name',
+               help=_('Logger name to use for this privsep context.  By '
+                      'default all contexts log with oslo_privsep.daemon.'),
+               default='oslo_privsep.daemon'),
 ]
 
 _ENTRYPOINT_ATTR = 'privsep_entrypoint'
@@ -121,9 +126,10 @@ def init(root_helper=None):
         _HELPER_COMMAND_PREFIX = root_helper
 
 
-class PrivContext(object):
+class PrivContext:
     def __init__(self, prefix, cfg_section='privsep', pypath=None,
-                 capabilities=None):
+                 capabilities=None, logger_name='oslo_privsep.daemon',
+                 timeout=None):
 
         # Note that capabilities=[] means retaining no capabilities
         # and leaves even uid=0 with no powers except being able to
@@ -144,10 +150,14 @@ class PrivContext(object):
         # The client_mode should be set to False on Windows.
         self.client_mode = sys.platform != 'win32'
         self.channel = None
+        self.start_lock = threading.Lock()
 
         cfg.CONF.register_opts(OPTS, group=cfg_section)
         cfg.CONF.set_default('capabilities', group=cfg_section,
                              default=capabilities)
+        cfg.CONF.set_default('logger_name', group=cfg_section,
+                             default=logger_name)
+        self.timeout = timeout
 
     @property
     def conf(self):
@@ -213,7 +223,22 @@ class PrivContext(object):
 
     def entrypoint(self, func):
         """This is intended to be used as a decorator."""
+        return self._entrypoint(func)
 
+    def entrypoint_with_timeout(self, timeout):
+        """This is intended to be used as a decorator with timeout."""
+
+        def wrap(func):
+
+            @functools.wraps(func)
+            def inner(*args, **kwargs):
+                f = self._entrypoint(func)
+                return f(*args, _wrap_timeout=timeout, **kwargs)
+            setattr(inner, _ENTRYPOINT_ATTR, self)
+            return inner
+        return wrap
+
+    def _entrypoint(self, func):
         if not func.__module__.startswith(self.prefix):
             raise AssertionError('%r entrypoints must be below "%s"' %
                                  (self, self.prefix))
@@ -234,31 +259,34 @@ class PrivContext(object):
     def is_entrypoint(self, func):
         return getattr(func, _ENTRYPOINT_ATTR, None) is self
 
-    def _wrap(self, func, *args, **kwargs):
+    def _wrap(self, func, *args, _wrap_timeout=None, **kwargs):
         if self.client_mode:
-            name = '%s.%s' % (func.__module__, func.__name__)
+            name = '{}.{}'.format(func.__module__, func.__name__)
             if self.channel is not None and not self.channel.running:
                 LOG.warning("RESTARTING PrivContext for %s", name)
                 self.stop()
             if self.channel is None:
                 self.start()
-            return self.channel.remote_call(name, args, kwargs)
+            r_call_timeout = _wrap_timeout or self.timeout
+            return self.channel.remote_call(name, args, kwargs,
+                                            r_call_timeout)
         else:
             return func(*args, **kwargs)
 
     def start(self, method=Method.ROOTWRAP):
-        if self.channel is not None:
-            LOG.warning('privsep daemon already running')
-            return
+        with self.start_lock:
+            if self.channel is not None:
+                LOG.warning('privsep daemon already running')
+                return
 
-        if method is Method.ROOTWRAP:
-            channel = daemon.RootwrapClientChannel(context=self)
-        elif method is Method.FORK:
-            channel = daemon.ForkingClientChannel(context=self)
-        else:
-            raise ValueError('Unknown method: %s' % method)
+            if method is Method.ROOTWRAP:
+                channel = daemon.RootwrapClientChannel(context=self)
+            elif method is Method.FORK:
+                channel = daemon.ForkingClientChannel(context=self)
+            else:
+                raise ValueError('Unknown method: %s' % method)
 
-        self.channel = channel
+            self.channel = channel
 
     def stop(self):
         if self.channel is not None:
