@@ -15,8 +15,8 @@
 
 import logging
 import os
-import pipes
 import platform
+import shlex
 import sys
 import tempfile
 import time
@@ -50,12 +50,12 @@ def do_some_long(long_timeout=0.4):
 
 class CustomError(Exception):
     def __init__(self, code, msg):
-        super(CustomError, self).__init__(code, msg)
+        super().__init__(code, msg)
         self.code = code
         self.msg = msg
 
     def __str__(self):
-        return 'Code %s: %s' % (self.code, self.msg)
+        return 'Code {}: {}'.format(self.code, self.msg)
 
 
 @testctx.context.entrypoint
@@ -172,7 +172,7 @@ class SeparationTest(testctx.TestContextTestCase):
                   'works only on Linux platform.')
 class RootwrapTest(testctx.TestContextTestCase):
     def setUp(self):
-        super(RootwrapTest, self).setUp()
+        super().setUp()
         testctx.context.stop()
 
         # Generate a command that will run daemon.helper_main without
@@ -186,7 +186,7 @@ class RootwrapTest(testctx.TestContextTestCase):
             cmd.append('--debug')
 
         self.privsep_conf.set_override(
-            'helper_command', ' '.join(map(pipes.quote, cmd)),
+            'helper_command', ' '.join(map(shlex.quote, cmd)),
             group=testctx.context.cfg_section)
 
         testctx.context.start(method=priv_context.Method.ROOTWRAP)
